'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _firebase = require('firebase');

var _firebase2 = _interopRequireDefault(_firebase);

var _database = require('firebase/database');

var _database2 = _interopRequireDefault(_database);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var HN_DATABASE_URL = 'https://hacker-news.firebaseio.com';
var HN_VERSION = 'v0';
var stamp = function stamp() {
	return (typeof window === 'undefined' ? 'undefined' : _typeof(window)) === 'object' ? performance.now() : Date.now();
};

var HNFirebaseCache = function () {
	function HNFirebaseCache() {
		_classCallCheck(this, HNFirebaseCache);

		this.reset();
	}

	_createClass(HNFirebaseCache, [{
		key: 'reset',
		value: function reset() {
			this._data = {
				items: {},
				top: [],
				new: [],
				best: [],
				ask: [],
				show: [],
				job: [],
				users: {}
			};
		}
	}, {
		key: 'touch',
		value: function touch(type) {
			this._data[type]._updated = stamp();
		}
	}, {
		key: 'set',
		value: function set(type, val) {
			if (Array.isArray(this._data[type])) {
				this._data[type] = val;
			} else if (_typeof(this._data[type]) === 'object') {
				this._data[type][val.id] = val;
			} else {
				throw new Error('Unsupported type ' + type);
			}

			this.touch(type);
		}
	}, {
		key: 'get',
		value: function get(type) {
			return this._data[type];
		}
	}, {
		key: 'exist',
		value: function exist(type) {
			if (Array.isArray(this._data[type])) {
				return this._data[type].length > 0;
			} else if (_typeof(this._data[type]) === 'object') {
				return Object.keys(this._data[type]) > 0;
			}
			throw new Error('Unsupported type ' + type);
		}
	}, {
		key: 'length',
		value: function length(type) {
			return Array.isArray(this._data[type]) ? this._data[type].length : Object.keys(this._data[type]).length;
		}
	}, {
		key: 'cached',
		value: function cached(id) {
			// @todo: we need to check timestamp
			return this._data.items[id];
		}
	}, {
		key: 'data',
		value: function data(_data) {
			if (_data) {
				this._data = _data;
			}

			return this._data;
		}
	}]);

	return HNFirebaseCache;
}();

var STORIES = ['top', 'new', 'best', 'ask', 'show', 'job'];

var HNFirebase = function () {
	function HNFirebase() {
		_classCallCheck(this, HNFirebase);

		this._app = _firebase2.default.initializeApp({ databaseURL: HN_DATABASE_URL }, 'other');
		this._database = this._app.database();
		this._cache = new HNFirebaseCache();
	}

	_createClass(HNFirebase, [{
		key: '_defaultOption',
		value: function _defaultOption(opts, more) {
			return Object.assign({
				page: 1,
				count: 30
			}, more, opts);
		}
	}, {
		key: '_fetch',
		value: function _fetch(param) {
			var _this = this;

			return new Promise(function (resolve, reject) {
				_this._app.database().ref(HN_VERSION + '/' + param).once('value', function (s) {
					resolve(s.val());
				}).catch(function (err) {
					console.error(err);
					reject(err);
				});
			});
		}
	}, {
		key: 'stories',
		value: function stories(type, opts) {
			var _this2 = this;

			if (!STORIES.includes(type)) {
				return new Error('Invalid type of stories ' + type);
			}

			opts = this._defaultOption(opts, { force: false });

			var fetch = function fetch() {
				return _this2._fetch(type + 'stories').then(function (items) {
					_this2._cache.set(type, items);
				});
			};

			return Promise.resolve(opts.force ? false : this._cache.exist(type)).then(function (cached) {
				return Promise.resolve(cached ? true : fetch());
			}).then(function () {
				return _this2._getItems(type, opts);
			});
		}
	}, {
		key: '_getItems',
		value: function _getItems(type, opts, sync) {
			var ids = this._cache.get(type);
			var begin = opts.page > 0 ? (opts.page - 1) * opts.count : 0;
			var end = opts.page > 0 ? begin + opts.count : ids.length;
			var items = ids.slice(begin, end);

			return sync === true ? this.itemsCached(items, opts) : this.items(items, opts);
		}
	}, {
		key: 'items',
		value: function items(ids) {
			var _this3 = this;

			var opts = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : { force: false };

			if (!Array.isArray(ids)) {
				ids = [ids];
			}

			return Promise.all(ids.map(function (id) {
				var item = opts.force ? undefined : _this3._cache.cached(id);
				return item ? Promise.resolve(item) : _this3._fetch('item/' + id);
			})).then(function (items) {
				items.forEach(function (i) {
					return _this3._cache.set('items', i);
				});
				return items;
			});
		}
	}, {
		key: 'user',
		value: function user(id) {
			var _this4 = this;

			var opts = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : { force: false };

			var fetch = function fetch() {
				return _this4._fetch('user/' + id).then(function (user) {
					return _this4._cache.set('users', user);
				});
			};

			return Promise.resolve(opts.force ? false : this._cache.exist('users')).then(function (cached) {
				return Promise.resolve(cached ? true : fetch());
			}).then(function () {
				return _this4._cache.get('users')[id];
			});
		}
	}, {
		key: 'maxItem',
		value: function maxItem() {
			return this._fetch('maxitem');
		}
	}, {
		key: 'update',
		value: function update() {
			var _this5 = this;

			return this._fetch('updates').then(function (updates) {
				var req = updates.profiles.map(function (id) {
					return _this5.user(id, true);
				}).concat(_this5.items(updates.items));

				return Promise.all(req).then(function () {
					return updates;
				});
			});
		}
	}, {
		key: 'watch',
		value: function watch() {
			var _this6 = this;

			var on = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : true;

			return new Promise(function (resolve) {
				var method = on ? 'on' : 'off';
				STORIES.forEach(function (type) {
					_this6._database.ref(HN_VERSION + '/' + type + 'stories')[method]('value', function (s) {
						_this6._cache.set(type, s.val());
						resolve();
					});
				});
			});
		}
	}, {
		key: 'length',
		value: function length(type) {
			var _this7 = this;

			return new Promise(function (resolve) {
				return resolve(_this7._cache.length(type));
			});
		}
	}, {
		key: 'kids',
		value: function kids(id) {
			var _this8 = this;

			var travelKids = function travelKids(ids) {
				if (ids && ids.length > 0) {
					return _this8.items(ids).then(function (items) {
						return Promise.all(items.map(function (i) {
							return travelKids(i.kids);
						}));
					});
				}
			};

			return travelKids(this._cache.cached(id).kids);
		}
	}, {
		key: 'cached',
		value: function cached(id) {
			console.error('cached method will be deprecated in favour of itemsCached');
			return this._cache.cached(id);
		}
	}, {
		key: 'data',
		value: function data(_data2) {
			var _this9 = this;

			return new Promise(function (resolve) {
				return resolve(_this9._cache.data(_data2));
			});
		}
	}, {
		key: 'itemsCached',
		value: function itemsCached(ids) {
			var _this10 = this;

			if (!Array.isArray(ids)) {
				ids = [ids];
			}

			return ids.map(function (id) {
				return _this10._cache.cached(id);
			}).filter(function (item) {
				return item !== undefined;
			});
		}
	}, {
		key: 'storiesCached',
		value: function storiesCached(type, opts) {
			if (!STORIES.includes(type)) {
				return new Error('Invalid type of stories ' + type);
			}

			return this._getItems(type, this._defaultOption(opts), true);
		}
	}, {
		key: 'lengthCached',
		value: function lengthCached(type) {
			return this._cache.length(type);
		}
	}, {
		key: 'dataCached',
		value: function dataCached(data) {
			return this._cache.data(data);
		}
	}]);

	return HNFirebase;
}();

exports.default = function () {
	var _app = void 0;

	function createService(opts) {
		if (!_app) {
			_app = new HNFirebase(opts);
		}

		return _app;
	}

	return createService;
}();
